@@ -7,9 +7,7 @@ import logging
 from datetime import datetime, timedelta
 import asyncio
 import re
-import json
 import pytz
-from bs4 import BeautifulSoup
 
 _logger = logging.getLogger(__name__)
 
@@ -22,8 +20,9 @@ DATA_SOURCES = {
         "name": "Published",
         "type": "nswcoviddate",
         "unit": "date",
+        "iconId": "mdi:calendar-range",
         "selector": "#maincontent > nav > h1",
-        "regex": "\d+[ap]m\s+\d+\s+(?:january|february|march|april|may|june|july|august|september|october|november|december)\s+(?:20)?\d\d",
+        "regex": "\\d+[ap]m\\s+\\d+\\s+(?:january|february|march|april|may|june|july|august|september|october|november|december)\\s+(?:20)?\\d\\d",
     },
     "locally_active": {
         "host": None,
@@ -31,6 +30,7 @@ DATA_SOURCES = {
         "name": "Locally Active",
         "type": "integer",
         "unit": "case",
+        "iconId": "mdi:virus",
         "selector": "#ContentHtml1Zone2 > div:nth-child(1) > div > div.active-cases.calloutbox > ul > li:nth-child(1) > span",
     },
     "interstate_active": {
@@ -39,6 +39,7 @@ DATA_SOURCES = {
         "name": "Interstate Active",
         "type": "integer",
         "unit": "case",
+        "iconId": "mdi:virus",
         "selector": "#ContentHtml1Zone2 > div:nth-child(1) > div > div.active-cases.calloutbox > ul > li:nth-child(2) > span",
     },
     "overseas_active": {
@@ -47,6 +48,7 @@ DATA_SOURCES = {
         "name": "Overseas Active",
         "type": "integer",
         "unit": "case",
+        "iconId": "mdi:virus",
         "selector": "#ContentHtml1Zone2 > div:nth-child(1) > div > div.active-cases.calloutbox > ul > li:nth-child(3) > span",
     },
     "last_24_hours_known": {
@@ -55,6 +57,7 @@ DATA_SOURCES = {
         "name": "Last 24 Hours Known Source",
         "type": "integer",
         "unit": "case",
+        "iconId": "mdi:virus",
         "selector": "#known > ul > li:nth-child(1) > span.number",
     },
     "last_24_hours_unknown": {
@@ -63,6 +66,7 @@ DATA_SOURCES = {
         "name": "Last 24 Hours Unknown Source",
         "type": "integer",
         "unit": "case",
+        "iconId": "mdi:virus",
         "selector": "#unknown > ul > li:nth-child(1) > span.number",
     },
     "last_24_hours_interstate": {
@@ -71,6 +75,7 @@ DATA_SOURCES = {
         "name": "Last 24 Hours Interstate Source",
         "type": "integer",
         "unit": "case",
+        "iconId": "mdi:virus",
         "selector": "#interstate > ul > li:nth-child(1) > span.number",
     },
     "last_24_hours_overseas": {
@@ -79,6 +84,7 @@ DATA_SOURCES = {
         "name": "Last 24 Hours Overseas Source",
         "type": "integer",
         "unit": "case",
+        "iconId": "mdi:virus",
         "selector": "#overseas > ul > li:nth-child(1) > span.number",
     },
     "last_24_hours_total": {
@@ -87,6 +93,7 @@ DATA_SOURCES = {
         "name": "Last 24 Hours Total",
         "type": "integer",
         "unit": "case",
+        "iconId": "mdi:virus",
         "selector": "#case > ul > li:nth-child(1) > span.number",
     },
     "last_24_hours_tests": {
@@ -95,6 +102,7 @@ DATA_SOURCES = {
         "name": "Last 24 Hours Tests",
         "type": "integer",
         "unit": "test",
+        "iconId": "mdi:test-tube",
         "selector": "#testing > ul > li:nth-child(1) > span.number",
     },
     "this_week_known": {
@@ -103,6 +111,7 @@ DATA_SOURCES = {
         "name": "This Week Known Source",
         "type": "integer",
         "unit": "case",
+        "iconId": "mdi:virus",
         "selector": "#known > ul > li:nth-child(2) > span.number",
     },
     "this_week_unknown": {
@@ -111,6 +120,7 @@ DATA_SOURCES = {
         "name": "This Week Unknown Source",
         "type": "integer",
         "unit": "case",
+        "iconId": "mdi:virus",
         "selector": "#unknown > ul > li:nth-child(2) > span.number",
     },
     "this_week_interstate": {
@@ -119,6 +129,7 @@ DATA_SOURCES = {
         "name": "This Week Interstate Source",
         "type": "integer",
         "unit": "case",
+        "iconId": "mdi:virus",
         "selector": "#interstate > ul > li:nth-child(2) > span.number",
     },
     "this_week_overseas": {
@@ -127,6 +138,7 @@ DATA_SOURCES = {
         "name": "This Week Overseas Source",
         "type": "integer",
         "unit": "case",
+        "iconId": "mdi:virus",
         "selector": "#overseas > ul > li:nth-child(2) > span.number",
     },
     "this_week_total": {
@@ -135,6 +147,7 @@ DATA_SOURCES = {
         "name": "This Week Total",
         "type": "integer",
         "unit": "case",
+        "iconId": "mdi:virus",
         "selector": "#case > ul > li:nth-child(2) > span.number",
     },
     "this_week_tests": {
@@ -143,6 +156,7 @@ DATA_SOURCES = {
         "name": "This Week Tests",
         "type": "integer",
         "unit": "test",
+        "iconId": "mdi:test-tube",
         "selector": "#testing > ul > li:nth-child(2) > span.number",
     },
     "last_week_known": {
@@ -151,6 +165,7 @@ DATA_SOURCES = {
         "name": "Last Week Known Source",
         "type": "integer",
         "unit": "case",
+        "iconId": "mdi:virus",
         "selector": "#known > ul > li:nth-child(3) > span.number",
     },
     "last_week_unknown": {
@@ -159,6 +174,7 @@ DATA_SOURCES = {
         "name": "Last Week Unknown Source",
         "type": "integer",
         "unit": "case",
+        "iconId": "mdi:virus",
         "selector": "#unknown > ul > li:nth-child(3) > span.number",
     },
     "last_week_interstate": {
@@ -167,6 +183,7 @@ DATA_SOURCES = {
         "name": "Last Week Interstate Source",
         "type": "integer",
         "unit": "case",
+        "iconId": "mdi:virus",
         "selector": "#interstate > ul > li:nth-child(3) > span.number",
     },
     "last_week_overseas": {
@@ -175,6 +192,7 @@ DATA_SOURCES = {
         "name": "Last Week Overseas Source",
         "type": "integer",
         "unit": "case",
+        "iconId": "mdi:virus",
         "selector": "#overseas > ul > li:nth-child(3) > span.number",
     },
     "last_week_total": {
@@ -183,6 +201,7 @@ DATA_SOURCES = {
         "name": "Last Week Total",
         "type": "integer",
         "unit": "case",
+        "iconId": "mdi:virus",
         "selector": "#case > ul > li:nth-child(3) > span.number",
     },
     "last_week_tests": {
@@ -191,6 +210,7 @@ DATA_SOURCES = {
         "name": "Last Week Tests",
         "type": "integer",
         "unit": "test",
+        "iconId": "mdi:test-tube",
         "selector": "#testing > ul > li:nth-child(3) > span.number",
     },
     "this_year_known": {
@@ -199,6 +219,7 @@ DATA_SOURCES = {
         "name": "This Year Known Source",
         "type": "integer",
         "unit": "case",
+        "iconId": "mdi:virus",
         "selector": "#known > ul > li:nth-child(4) > span.number",
     },
     "this_year_unknown": {
@@ -207,6 +228,7 @@ DATA_SOURCES = {
         "name": "This Year Unknown Source",
         "type": "integer",
         "unit": "case",
+        "iconId": "mdi:virus",
         "selector": "#unknown > ul > li:nth-child(4) > span.number",
     },
     "this_year_interstate": {
@@ -215,6 +237,7 @@ DATA_SOURCES = {
         "name": "This Year Interstate Source",
         "type": "integer",
         "unit": "case",
+        "iconId": "mdi:virus",
         "selector": "#interstate > ul > li:nth-child(4) > span.number",
     },
     "this_year_overseas": {
@@ -223,6 +246,7 @@ DATA_SOURCES = {
         "name": "This Year Overseas Source",
         "type": "integer",
         "unit": "case",
+        "iconId": "mdi:virus",
         "selector": "#overseas > ul > li:nth-child(4) > span.number",
     },
     "this_year_total": {
@@ -231,6 +255,7 @@ DATA_SOURCES = {
         "name": "This Year Total",
         "type": "integer",
         "unit": "case",
+        "iconId": "mdi:virus",
         "selector": "#case > ul > li:nth-child(4) > span.number",
     },
     "this_year_tests": {
@@ -239,6 +264,7 @@ DATA_SOURCES = {
         "name": "This Year Tests",
         "type": "integer",
         "unit": "test",
+        "iconId": "mdi:test-tube",
         "selector": "#testing > ul > li:nth-child(4) > span.number",
     },
     "last_24_hours_first_dose": {
@@ -247,7 +273,8 @@ DATA_SOURCES = {
         "name": "Last 24 Hours First Dose Vaccine",
         "type": "integer",
         "unit": "dose",
-        "selector": "#ContentHtml1Zone2 > div:nth-child(3) > div > table > tbody > tr:nth-child(2) > td:nth-child(2)",
+        "iconId": "mdi:needle",
+        "xpath": '//*[@id="ContentHtml1Zone2"]/div[3]/div/table/tbody/tr[2]/td[2]/text()',
     },
     "last_24_hours_second_dose": {
         "host": None,
@@ -255,7 +282,8 @@ DATA_SOURCES = {
         "name": "Last 24 Hours Second Dose Vaccine",
         "type": "integer",
         "unit": "dose",
-        "selector": "#ContentHtml1Zone2 > div:nth-child(3) > div > table > tbody > tr:nth-child(3) > td:nth-child(2)",
+        "iconId": "mdi:needle",
+        "xpath": '//*[@id="ContentHtml1Zone2"]/div[3]/div/table/tbody/tr[3]/td[2]/text()',
     },
     "last_24_hours_total_dose": {
         "host": None,
@@ -263,7 +291,8 @@ DATA_SOURCES = {
         "name": "Last 24 Hours Vaccine Total",
         "type": "integer",
         "unit": "dose",
-        "selector": "#ContentHtml1Zone2 > div:nth-child(3) > div > table > tbody > tr:nth-child(4) > td:nth-child(2)",
+        "iconId": "mdi:needle",
+        "xpath": '//*[@id="ContentHtml1Zone2"]/div[3]/div/table/tbody/tr[4]/td[2]/text()',
     },
     "total_first_dose": {
         "host": None,
@@ -271,7 +300,8 @@ DATA_SOURCES = {
         "name": "Total First Dose Vaccine",
         "type": "integer",
         "unit": "dose",
-        "selector": "#ContentHtml1Zone2 > div:nth-child(3) > div > table > tbody > tr:nth-child(2) > td:nth-child(3)",
+        "iconId": "mdi:needle",
+        "xpath": '//*[@id="ContentHtml1Zone2"]/div[3]/div/table/tbody/tr[2]/td[3]/text()',
     },
     "total_second_dose": {
         "host": None,
@@ -279,7 +309,8 @@ DATA_SOURCES = {
         "name": "Total Second Dose Vaccine",
         "type": "integer",
         "unit": "dose",
-        "selector": "#ContentHtml1Zone2 > div:nth-child(3) > div > table > tbody > tr:nth-child(3) > td:nth-child(3)",
+        "iconId": "mdi:needle",
+        "xpath": '//*[@id="ContentHtml1Zone2"]/div[3]/div/table/tbody/tr[3]/td[3]/text()',
     },
     "total_total_dose": {
         "host": None,
@@ -287,9 +318,12 @@ DATA_SOURCES = {
         "name": "Total Vaccine Doses",
         "type": "integer",
         "unit": "dose",
-        "selector": "#ContentHtml1Zone2 > div:nth-child(3) > div > table > tbody > tr:nth-child(4) > td:nth-child(3)",
+        "iconId": "mdi:needle",
+        "xpath": '//*[@id="ContentHtml1Zone2"]/div[3]/div/table/tbody/tr[4]/td[3]/text()',
     },
 }
+# ENABLED_DATA_SOURCES = ["published", "total_first_dose"]
+ENABLED_DATA_SOURCES = list()
 
 
 class StatisticHandler(object):
@@ -323,9 +357,15 @@ class StatisticHandler(object):
             for statistic_reference in all_statistics:
                 if "id" in statistic_reference:
                     statistic_id = statistic_reference["id"]
-                    self.__statistics[statistic_id] = Statistic(
-                        handler=self, id=statistic_id, data=statistic_reference
-                    )
+                    if len(ENABLED_DATA_SOURCES) > 0:
+                        if statistic_id in ENABLED_DATA_SOURCES:
+                            self.__statistics[statistic_id] = Statistic(
+                                handler=self, id=statistic_id, data=statistic_reference
+                            )
+                    else:
+                        self.__statistics[statistic_id] = Statistic(
+                            handler=self, id=statistic_id, data=statistic_reference
+                        )
             page += 1
             all_statistics = await self.__listall(limit=limit, page=page)
         statistic_ids = self.__statistics.keys()
@@ -380,9 +420,13 @@ class StatisticHandler(object):
 
         value = None
 
-        if hasattr(statistic, "selector") and statistic.selector is not None:
-            soup = BeautifulSoup(body, features="lxml")
-            reference = soup.select_one(statistic.selector)
+        if (
+            hasattr(statistic, "selector")
+            and statistic.selector is not None
+            and get["soup"] is not None
+        ):
+            reference = get["soup"].select_one(statistic.selector)
+            _logger.debug("%s :: %s", statistic.selector, reference)
 
             if not reference:
                 return statistic
@@ -392,6 +436,22 @@ class StatisticHandler(object):
             if not value:
                 return statistic
 
+        if (
+            hasattr(statistic, "xpath")
+            and statistic.xpath is not None
+            and get["dom"] is not None
+        ):
+            reference = get["dom"].xpath(statistic.xpath)
+            _logger.debug("%s :: %s", statistic.xpath, reference)
+
+            if not reference and reference[0]:
+                return statistic
+
+            value = reference[0]
+
+            if not value:
+                return statistic
+
         if hasattr(statistic, "regex") and statistic.regex is not None:
             match = statistic.regex.search(str(value))
             if not match:
@@ -488,6 +548,8 @@ class Statistic(object):
                 self.__unit = data["unit"]
             if "selector" in data:
                 self.__selector = data["selector"]
+            if "xpath" in data:
+                self.__xpath = data["xpath"]
             if "regex" in data:
                 self.__regex = data["regex"]
             if "typeId" in data:
@@ -513,7 +575,6 @@ class Statistic(object):
     async def refresh(self, event_receiver=None):
         if not self.__id:
             return
-        _logger.debug("Updating statistic %s value", self.__id)
         await self.__handler.details(self.__id)
         if self.changed and event_receiver is not None:
             try:
@@ -559,6 +620,7 @@ class Statistic(object):
             self.__changed = self.__check_changed()
         except AttributeError:
             pass
+        _logger.debug("Updating statistic %s: %s", self.__id, self.status)
 
     @property
     def attribution(self):
@@ -652,6 +714,13 @@ class Statistic(object):
         except AttributeError:
             return None
 
+    @property
+    def xpath(self):
+        try:
+            return self.__xpath
+        except AttributeError:
+            return None
+
     @property
     def regex(self):
         regex = None
